@@ -10,6 +10,7 @@ from django.contrib import messages
 from django.contrib.auth import (
     authenticate,
     get_backends,
+    get_user_model,
     login as django_login,
     logout as django_logout,
 )
@@ -28,13 +29,16 @@ from django.utils.crypto import get_random_string
 from django.utils.encoding import force_str
 from django.utils.translation import gettext_lazy as _
 
-from allauth import ratelimit
+from allauth import app_settings as allauth_app_settings
 from allauth.account import signals
-from allauth.account.app_settings import EmailVerificationMethod
+from allauth.account.app_settings import (
+    AuthenticationMethod,
+    EmailVerificationMethod,
+)
+from allauth.core import context, ratelimit
 from allauth.utils import (
     build_absolute_uri,
     generate_unique_username,
-    get_user_model,
     import_attribute,
 )
 
@@ -53,10 +57,13 @@ class DefaultAccountAdapter(object):
             "Too many failed login attempts. Try again later."
         ),
         "email_taken": _("A user is already registered with this email address."),
+        "incorrect_password": _("Incorrect password."),
     }
 
     def __init__(self, request=None):
-        self.request = request
+        # Explicitly passing `request` is deprecated, just use:
+        # `allauth.core.context.request`.
+        self.request = context.request
 
     def stash_verified_email(self, request, email):
         request.session["account_verified_email"] = email
@@ -84,10 +91,39 @@ class DefaultAccountAdapter(object):
             ret = verified_email.lower() == email.lower()
         return ret
 
+    def can_delete_email(self, email_address):
+        from allauth.account.models import EmailAddress
+
+        has_other = (
+            EmailAddress.objects.filter(user_id=email_address.user_id)
+            .exclude(pk=email_address.pk)
+            .exists()
+        )
+        login_by_email = (
+            app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.EMAIL
+        )
+        if email_address.primary:
+            if has_other:
+                # Don't allow, let the user mark one of the others as primary
+                # first.
+                return False
+            elif login_by_email:
+                # Last email & login is by email, prevent dangling account.
+                return False
+            return True
+        elif has_other:
+            # Account won't be dangling.
+            return True
+        elif login_by_email:
+            # This is the last email.
+            return False
+        else:
+            return True
+
     def format_email_subject(self, subject):
         prefix = app_settings.EMAIL_SUBJECT_PREFIX
         if prefix is None:
-            site = get_current_site(self.request)
+            site = get_current_site(context.request)
             prefix = "[{name}] ".format(name=site.name)
         return prefix + force_str(subject)
 
@@ -118,7 +154,7 @@ class DefaultAccountAdapter(object):
                 bodies[ext] = render_to_string(
                     template_name,
                     context,
-                    self.request,
+                    globals()["context"].request,
                 ).strip()
             except TemplateDoesNotExist:
                 if ext == "txt" and not bodies:
@@ -307,10 +343,6 @@ class DefaultAccountAdapter(object):
         return password
 
     def validate_unique_email(self, email):
-        from .models import EmailAddress
-
-        if EmailAddress.objects.is_verified(email):
-            raise forms.ValidationError(self.error_messages["email_taken"])
         return email
 
     def add_message(
@@ -332,7 +364,7 @@ class DefaultAccountAdapter(object):
                 escaped_message = render_to_string(
                     message_template,
                     message_context,
-                    self.request,
+                    context.request,
                 ).strip()
                 if escaped_message:
                     message = html.unescape(escaped_message)
@@ -449,6 +481,8 @@ class DefaultAccountAdapter(object):
         return response
 
     def login(self, request, user):
+        from allauth.account.reauthentication import record_authentication
+
         # HACK: This is not nice. The proper Django way is to use an
         # authentication backend
         if not hasattr(user, "backend"):
@@ -467,6 +501,7 @@ class DefaultAccountAdapter(object):
             backend_path = ".".join([backend.__module__, backend.__class__.__name__])
             user.backend = backend_path
         django_login(request, user)
+        record_authentication(request, user)
 
     def logout(self, request):
         django_logout(request)
@@ -522,15 +557,10 @@ class DefaultAccountAdapter(object):
         return ret
 
     def is_safe_url(self, url):
-        try:
-            from django.utils.http import url_has_allowed_host_and_scheme
-        except ImportError:
-            from django.utils.http import (
-                is_safe_url as url_has_allowed_host_and_scheme,
-            )
+        from django.utils.http import url_has_allowed_host_and_scheme
 
         # get_host already validates the given host, so no need to check it again
-        allowed_hosts = {self.request.get_host()} | set(settings.ALLOWED_HOSTS)
+        allowed_hosts = {context.request.get_host()} | set(settings.ALLOWED_HOSTS)
 
         if "*" in allowed_hosts:
             parsed_host = urlparse(url).netloc
@@ -570,18 +600,18 @@ class DefaultAccountAdapter(object):
         return send_email
 
     def send_account_already_exists_mail(self, email):
-        signup_url = build_absolute_uri(self.request, reverse("account_signup"))
+        signup_url = build_absolute_uri(context.request, reverse("account_signup"))
         password_reset_url = build_absolute_uri(
-            self.request, reverse("account_reset_password")
+            context.request, reverse("account_reset_password")
         )
-        context = {
-            "request": self.request,
-            "current_site": get_current_site(self.request),
+        ctx = {
+            "request": context.request,
+            "current_site": get_current_site(context.request),
             "email": email,
             "signup_url": signup_url,
             "password_reset_url": password_reset_url,
         }
-        self.send_mail("account/email/account_already_exists", email, context)
+        self.send_mail("account/email/account_already_exists", email, ctx)
 
     def send_confirmation_mail(self, request, emailconfirmation, signup):
         current_site = get_current_site(request)
@@ -667,6 +697,12 @@ class DefaultAccountAdapter(object):
         key = get_random_string(64).lower()
         return key
 
+    def get_login_stages(self):
+        ret = []
+        if allauth_app_settings.MFA_ENABLED:
+            ret.append("allauth.mfa.stages.AuthenticateStage")
+        return ret
+
 
 def get_adapter(request=None):
     return import_attribute(app_settings.ADAPTER)(request)
