@@ -1,12 +1,12 @@
-from __future__ import absolute_import
-
 from importlib import import_module
 
 from django import forms
+from django.contrib.auth import password_validation
 from django.contrib.auth.tokens import PasswordResetTokenGenerator
 from django.contrib.sites.shortcuts import get_current_site
 from django.core import exceptions, validators
-from django.urls import reverse
+from django.urls import NoReverseMatch, reverse
+from django.utils.safestring import mark_safe
 from django.utils.translation import gettext, gettext_lazy as _, pgettext
 
 from ..utils import (
@@ -19,6 +19,7 @@ from .adapter import get_adapter
 from .app_settings import AuthenticationMethod
 from .models import EmailAddress
 from .utils import (
+    assess_unique_email,
     filter_users_by_email,
     get_user_model,
     perform_login,
@@ -137,6 +138,15 @@ class LoginForm(forms.Form):
         set_form_field_order(self, ["login", "password", "remember"])
         if app_settings.SESSION_REMEMBER is not None:
             del self.fields["remember"]
+        try:
+            reset_url = reverse("account_reset_password")
+        except NoReverseMatch:
+            pass
+        else:
+            forgot_txt = _("Forgot your password?")
+            self.fields["password"].help_text = mark_safe(
+                f'<a href="{reset_url}">{forgot_txt}</a>'
+            )
 
     def user_credentials(self):
         """
@@ -279,13 +289,6 @@ class BaseSignupForm(_base_signup_form_class()):
         self.username_required = kwargs.pop(
             "username_required", app_settings.USERNAME_REQUIRED
         )
-        # We can only truly prevent enumeration if email verification is
-        # mandatory, because in that case, regardless of whether or not the
-        # email is in use, the user will always receive an email.
-        self.prevent_enumeration = app_settings.PREVENT_ENUMERATION and (
-            app_settings.EMAIL_VERIFICATION
-            == app_settings.EmailVerificationMethod.MANDATORY
-        )
         self.account_already_exists = False
         super(BaseSignupForm, self).__init__(*args, **kwargs)
         username_field = self.fields["username"]
@@ -348,16 +351,22 @@ class BaseSignupForm(_base_signup_form_class()):
         value = self.cleaned_data["email"]
         value = get_adapter().clean_email(value)
         if value and app_settings.UNIQUE_EMAIL:
-            try:
-                value = self.validate_unique_email(value)
-            except forms.ValidationError:
-                if not self.prevent_enumeration:
-                    raise
-                self.account_already_exists = True
+            value = self.validate_unique_email(value)
         return value
 
     def validate_unique_email(self, value):
-        return get_adapter().validate_unique_email(value)
+        adapter = get_adapter()
+        assessment = assess_unique_email(value)
+        if assessment is True:
+            # All good.
+            pass
+        elif assessment is False:
+            # Fail right away.
+            raise forms.ValidationError(adapter.error_messages["email_taken"])
+        else:
+            assert assessment is None
+            self.account_already_exists = True
+        return adapter.validate_unique_email(value)
 
     def clean(self):
         cleaned_data = super(BaseSignupForm, self).clean()
@@ -377,10 +386,10 @@ class BaseSignupForm(_base_signup_form_class()):
         email and return a standard "email verification sent" response.
         """
         if self.account_already_exists:
-            # Don't create a new acount, only send an email informing the user
+            # Don't create a new account, only send an email informing the user
             # that (s)he already has one...
             email = self.cleaned_data["email"]
-            adapter = get_adapter(request)
+            adapter = get_adapter()
             adapter.send_account_already_exists_mail(email)
             user = None
             resp = adapter.respond_email_verification_sent(request, None)
@@ -394,7 +403,9 @@ class SignupForm(BaseSignupForm):
     def __init__(self, *args, **kwargs):
         super(SignupForm, self).__init__(*args, **kwargs)
         self.fields["password1"] = PasswordField(
-            label=_("Password"), autocomplete="new-password"
+            label=_("Password"),
+            autocomplete="new-password",
+            help_text=password_validation.password_validators_help_text_html(),
         )
         if app_settings.SIGNUP_PASSWORD_ENTER_TWICE:
             self.fields["password2"] = PasswordField(
@@ -434,14 +445,15 @@ class SignupForm(BaseSignupForm):
         return self.cleaned_data
 
     def save(self, request):
+        email = self.cleaned_data.get("email")
         if self.account_already_exists:
-            raise ValueError(self.cleaned_data.get("email"))
-        adapter = get_adapter(request)
+            raise ValueError(email)
+        adapter = get_adapter()
         user = adapter.new_user(request)
         adapter.save_user(request, user, self)
         self.custom_signup(request, user)
         # TODO: Move into adapter `save_user` ?
-        setup_user_email(request, user, [])
+        setup_user_email(request, user, [EmailAddress(email=email)] if email else [])
         return user
 
 
@@ -461,8 +473,11 @@ class AddEmailForm(UserForm):
     )
 
     def clean_email(self):
+        from allauth.account import signals
+
         value = self.cleaned_data["email"]
-        value = get_adapter().clean_email(value)
+        adapter = get_adapter()
+        value = adapter.clean_email(value)
         errors = {
             "this_account": _(
                 "This email address is already associated with this account."
@@ -471,13 +486,26 @@ class AddEmailForm(UserForm):
         }
         users = filter_users_by_email(value)
         on_this_account = [u for u in users if u.pk == self.user.pk]
+        on_diff_account = [u for u in users if u.pk != self.user.pk]
 
         if on_this_account:
             raise forms.ValidationError(errors["this_account"])
+        if (
+            on_diff_account
+            and app_settings.PREVENT_ENUMERATION != "strict"
+            and app_settings.UNIQUE_EMAIL
+        ):
+            raise forms.ValidationError(adapter.error_messages["email_taken"])
         if not EmailAddress.objects.can_add_email(self.user):
             raise forms.ValidationError(
                 errors["max_email_addresses"] % app_settings.MAX_EMAIL_ADDRESSES
             )
+
+        signals._add_email.send(
+            sender=self.user.__class__,
+            email=value,
+            user=self.user,
+        )
         return value
 
     def save(self, request):
@@ -494,7 +522,10 @@ class ChangePasswordForm(PasswordVerificationMixin, UserForm):
     oldpassword = PasswordField(
         label=_("Current Password"), autocomplete="current-password"
     )
-    password1 = SetPasswordField(label=_("New Password"))
+    password1 = SetPasswordField(
+        label=_("New Password"),
+        help_text=password_validation.password_validators_help_text_html(),
+    )
     password2 = PasswordField(label=_("New Password (again)"))
 
     def __init__(self, *args, **kwargs):
@@ -511,7 +542,10 @@ class ChangePasswordForm(PasswordVerificationMixin, UserForm):
 
 
 class SetPasswordForm(PasswordVerificationMixin, UserForm):
-    password1 = SetPasswordField(label=_("Password"))
+    password1 = SetPasswordField(
+        label=_("Password"),
+        help_text=password_validation.password_validators_help_text_html(),
+    )
     password2 = PasswordField(label=_("Password (again)"))
 
     def __init__(self, *args, **kwargs):
@@ -561,7 +595,7 @@ class ResetPasswordForm(forms.Form):
             "request": request,
             "signup_url": signup_url,
         }
-        get_adapter(request).send_mail("account/email/unknown_account", email, context)
+        get_adapter().send_mail("account/email/unknown_account", email, context)
 
     def _send_password_reset_mail(self, request, email, users, **kwargs):
         token_generator = kwargs.get("token_generator", default_token_generator)
@@ -592,9 +626,7 @@ class ResetPasswordForm(forms.Form):
 
             if app_settings.AUTHENTICATION_METHOD != AuthenticationMethod.EMAIL:
                 context["username"] = user_username(user)
-            get_adapter(request).send_mail(
-                "account/email/password_reset_key", email, context
-            )
+            get_adapter().send_mail("account/email/password_reset_key", email, context)
 
 
 class ResetPasswordKeyForm(PasswordVerificationMixin, forms.Form):
@@ -646,3 +678,19 @@ class UserTokenForm(forms.Form):
             raise forms.ValidationError(self.error_messages["token_invalid"])
 
         return cleaned_data
+
+
+class ReauthenticateForm(forms.Form):
+    password = PasswordField(label=_("Password"), autocomplete="current-password")
+
+    def __init__(self, *args, **kwargs):
+        self.user = kwargs.pop("user")
+        super().__init__(*args, **kwargs)
+
+    def clean_password(self):
+        password = self.cleaned_data.get("password")
+        if not self.user.check_password(password):
+            raise forms.ValidationError(
+                get_adapter().error_messages["incorrect_password"]
+            )
+        return password
