@@ -1,22 +1,23 @@
 import datetime
 
+from django.conf import settings
+from django.contrib.auth import get_user_model
 from django.core import signing
 from django.db import models
-from django.db.models import Q
+from django.db.models import Index, Q
 from django.db.models.constraints import UniqueConstraint
+from django.db.models.functions import Upper
 from django.utils import timezone
 from django.utils.translation import gettext_lazy as _
 
-from .. import app_settings as allauth_app_settings
 from . import app_settings, signals
 from .adapter import get_adapter
 from .managers import EmailAddressManager, EmailConfirmationManager
-from .utils import user_email
 
 
 class EmailAddress(models.Model):
     user = models.ForeignKey(
-        allauth_app_settings.USER_MODEL,
+        settings.AUTH_USER_MODEL,
         verbose_name=_("user"),
         on_delete=models.CASCADE,
     )
@@ -41,6 +42,7 @@ class EmailAddress(models.Model):
                     condition=Q(verified=True),
                 )
             ]
+        indexes = [Index(Upper("email"), name="account_emailaddress_upper")]
 
     def __str__(self):
         return self.email
@@ -70,6 +72,8 @@ class EmailAddress(models.Model):
         """Marks the email address as primary. In case of `conditional`, it is
         only marked as primary if there is no other primary email address set.
         """
+        from allauth.account.utils import user_email
+
         old_primary = EmailAddress.objects.get_primary(self.user)
         if old_primary:
             if conditional:
@@ -90,6 +94,8 @@ class EmailAddress(models.Model):
         return confirmation
 
     def remove(self):
+        from allauth.account.utils import user_email
+
         self.delete()
         if user_email(self.user) == self.email:
             alt = (
@@ -107,7 +113,7 @@ class EmailConfirmationMixin:
     def confirm(self, request):
         email_address = self.email_address
         if not email_address.verified:
-            confirmed = get_adapter(request).confirm_email(request, email_address)
+            confirmed = get_adapter().confirm_email(request, email_address)
             if confirmed:
                 signals.email_confirmed.send(
                     sender=self.__class__,
@@ -117,7 +123,7 @@ class EmailConfirmationMixin:
                 return email_address
 
     def send(self, request=None, signup=False):
-        get_adapter(request).send_confirmation_mail(request, self, signup)
+        get_adapter().send_confirmation_mail(request, self, signup)
         signals.email_confirmation_sent.send(
             sender=self.__class__,
             request=request,
@@ -189,3 +195,81 @@ class EmailConfirmationHMAC(EmailConfirmationMixin, object):
         ):
             ret = None
         return ret
+
+
+class Login:
+    """
+    Represents a user that is in the process of logging in.
+    """
+
+    def __init__(
+        self,
+        user,
+        email_verification,
+        redirect_url=None,
+        signal_kwargs=None,
+        signup=False,
+        email=None,
+        state=None,
+    ):
+        self.user = user
+        self.email_verification = email_verification
+        self.redirect_url = redirect_url
+        self.signal_kwargs = signal_kwargs
+        self.signup = signup
+        self.email = email
+        self.state = {} if state is None else state
+
+    def serialize(self):
+        from allauth.account.utils import user_pk_to_url_str
+
+        # :-( Knowledge of the `socialaccount` is entering the `account` app.
+        signal_kwargs = self.signal_kwargs
+        if signal_kwargs is not None:
+            sociallogin = signal_kwargs.get("sociallogin")
+            if sociallogin is not None:
+                signal_kwargs = signal_kwargs.copy()
+                signal_kwargs["sociallogin"] = sociallogin.serialize()
+
+        data = {
+            "user_pk": user_pk_to_url_str(self.user),
+            "email_verification": self.email_verification,
+            "signup": self.signup,
+            "redirect_url": self.redirect_url,
+            "email": self.email,
+            "signal_kwargs": signal_kwargs,
+            "state": self.state,
+        }
+        return data
+
+    @classmethod
+    def deserialize(cls, data):
+        from allauth.account.utils import url_str_to_user_pk
+        from allauth.socialaccount.models import SocialLogin
+
+        user = (
+            get_user_model()
+            .objects.filter(pk=url_str_to_user_pk(data["user_pk"]))
+            .first()
+        )
+        if user is None:
+            raise ValueError()
+        try:
+            # :-( Knowledge of the `socialaccount` is entering the `account` app.
+            signal_kwargs = data["signal_kwargs"]
+            if signal_kwargs is not None:
+                sociallogin = signal_kwargs.get("sociallogin")
+                if sociallogin is not None:
+                    signal_kwargs = signal_kwargs.copy()
+                    signal_kwargs["sociallogin"] = SocialLogin.deserialize(sociallogin)
+
+            return Login(
+                user=user,
+                email_verification=data["email_verification"],
+                redirect_url=data["redirect_url"],
+                signup=data["signup"],
+                signal_kwargs=signal_kwargs,
+                state=data["state"],
+            )
+        except KeyError:
+            raise ValueError()
