@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+import warnings
 
 from django.core.exceptions import (
     ImproperlyConfigured,
@@ -9,6 +9,8 @@ from django.db.models import Q
 from django.urls import reverse
 from django.utils.translation import gettext_lazy as _
 
+from allauth.core import context
+
 from ..account.adapter import get_adapter as get_account_adapter
 from ..account.app_settings import EmailVerificationMethod
 from ..account.models import EmailAddress
@@ -32,7 +34,9 @@ class DefaultSocialAccountAdapter(object):
     }
 
     def __init__(self, request=None):
-        self.request = request
+        # Explicitly passing `request` is deprecated, just use:
+        # `allauth.core.context.request`.
+        self.request = context.request
 
     def pre_social_login(self, request, sociallogin):
         """
@@ -251,11 +255,13 @@ class DefaultSocialAccountAdapter(object):
                     "client_id",
                     "secret",
                     "key",
-                    "certificate_key",
                     "settings",
                 ]:
                     if field in config:
                         setattr(app, field, config[field])
+                if "certificate_key" in config:
+                    warnings.warn("'certificate_key' should be moved into app.settings")
+                    app.settings["certificate_key"] = config["certificate_key"]
                 if client_id and app.client_id != client_id:
                     continue
                 if (
