@@ -1,6 +1,5 @@
-from __future__ import absolute_import
-
-from django.contrib.auth import authenticate
+from django.conf import settings
+from django.contrib.auth import authenticate, get_user_model
 from django.contrib.sites.shortcuts import get_current_site
 from django.core.exceptions import PermissionDenied
 from django.db import models
@@ -9,13 +8,17 @@ from django.utils.translation import gettext_lazy as _
 
 import allauth.app_settings
 from allauth.account.models import EmailAddress
-from allauth.account.utils import get_next_redirect_url, setup_user_email
-from allauth.utils import get_user_model
+from allauth.account.utils import (
+    filter_users_by_email,
+    get_next_redirect_url,
+    setup_user_email,
+)
+from allauth.core import context
+from allauth.socialaccount import signals
 
 from ..utils import get_request_param
 from . import app_settings, providers
 from .adapter import get_adapter
-from .fields import JSONField
 
 
 class SocialAppManager(models.Manager):
@@ -68,13 +71,6 @@ class SocialApp(models.Model):
         # blank=True allows for disabling apps without removing them
         sites = models.ManyToManyField("sites.Site", blank=True)
 
-    # We want to move away from storing secrets in the database. So, we're
-    # putting a halt towards adding more fields for additional secrets, such as
-    # the certificate some providers need. Therefore, the certificate is not a
-    # DB backed field and can only be set using the ``APP`` configuration key
-    # in the provider settings.
-    certificate_key = None
-
     class Meta:
         verbose_name = _("social application")
         verbose_name_plural = _("social applications")
@@ -88,7 +84,7 @@ class SocialApp(models.Model):
 
 
 class SocialAccount(models.Model):
-    user = models.ForeignKey(allauth.app_settings.USER_MODEL, on_delete=models.CASCADE)
+    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
     # Given a `SocialApp` from which this account originates, this field equals
     # the app's `app.provider_id` if available, `app.provider` otherwise.
     provider = models.CharField(
@@ -116,7 +112,7 @@ class SocialAccount(models.Model):
     )
     last_login = models.DateTimeField(verbose_name=_("last login"), auto_now=True)
     date_joined = models.DateTimeField(verbose_name=_("date joined"), auto_now_add=True)
-    extra_data = JSONField(verbose_name=_("extra data"), default=dict)
+    extra_data = models.JSONField(verbose_name=_("extra data"), default=dict)
 
     class Meta:
         unique_together = ("provider", "uid")
@@ -141,7 +137,7 @@ class SocialAccount(models.Model):
         provider = getattr(self, "_provider", None)
         if provider:
             return provider
-        adapter = get_adapter(request)
+        adapter = get_adapter()
         provider = self._provider = adapter.get_provider(
             request, provider=self.provider
         )
@@ -152,7 +148,7 @@ class SocialAccount(models.Model):
 
 
 class SocialToken(models.Model):
-    app = models.ForeignKey(SocialApp, on_delete=models.CASCADE)
+    app = models.ForeignKey(SocialApp, on_delete=models.SET_NULL, blank=True, null=True)
     account = models.ForeignKey(SocialAccount, on_delete=models.CASCADE)
     token = models.TextField(
         verbose_name=_("token"),
@@ -214,6 +210,9 @@ class SocialLogin(object):
     def connect(self, request, user):
         self.user = user
         self.save(request, connect=True)
+        signals.social_account_added.send(
+            sender=SocialLogin, request=request, sociallogin=self
+        )
 
     def serialize(self):
         serialize_instance = get_adapter().serialize_instance
@@ -257,7 +256,7 @@ class SocialLogin(object):
         user.save()
         self.account.user = user
         self.account.save()
-        if app_settings.STORE_TOKENS and self.token and self.token.app_id:
+        if app_settings.STORE_TOKENS and self.token:
             self.token.account = self.account
             self.token.save()
         if connect:
@@ -271,15 +270,18 @@ class SocialLogin(object):
         """When `False`, this social login represents a temporary account, not
         yet backed by a database record.
         """
-        return self.user.pk is not None
+        if self.user.pk is None:
+            return False
+        return get_user_model().objects.filter(pk=self.user.pk).exists()
 
     def lookup(self):
         """Look up the existing local user account to which this social login
         points, if any.
         """
         if not self._lookup_by_socialaccount():
+            provider_id = self.account.get_provider().id
             if app_settings.EMAIL_AUTHENTICATION or app_settings.PROVIDERS.get(
-                self.account.provider, {}
+                provider_id, {}
             ).get("EMAIL_AUTHENTICATION", False):
                 self._lookup_by_email()
 
@@ -294,8 +296,11 @@ class SocialLogin(object):
             self.account = a
             self.user = self.account.user
             a.save()
+            signals.social_account_updated.send(
+                sender=SocialLogin, request=context.request, sociallogin=self
+            )
             # Update token
-            if app_settings.STORE_TOKENS and self.token and self.token.app.pk:
+            if app_settings.STORE_TOKENS and self.token:
                 assert not self.token.pk
                 try:
                     t = SocialToken.objects.get(
@@ -318,13 +323,13 @@ class SocialLogin(object):
 
     def _lookup_by_email(self):
         emails = [e.email for e in self.email_addresses if e.verified]
-        if not emails:
-            return
-        address = (
-            EmailAddress.objects.lookup(emails).order_by("-verified", "user_id").first()
-        )
-        if address:
-            self.user = address.user
+        for email in emails:
+            users = filter_users_by_email(email, is_active=True, prefer_verified=True)
+            if users:
+                self.user = users[0]
+                if app_settings.EMAIL_AUTHENTICATION_AUTO_CONNECT:
+                    self.connect(context.request, self.user)
+                return
 
     def get_redirect_url(self, request):
         url = self.state.get("next")
