@@ -1,5 +1,6 @@
 from django.shortcuts import render
 
+from allauth.account import app_settings as account_app_settings
 from allauth.socialaccount import app_settings
 
 
@@ -9,7 +10,7 @@ class OAuthLoginMixin:
         if (not app_settings.LOGIN_ON_GET) and request.method == "GET":
             return render(
                 request,
-                "socialaccount/login.html",
+                "socialaccount/login." + account_app_settings.TEMPLATE_EXTENSION,
                 {
                     "provider": provider,
                     "process": request.GET.get("process"),
