@@ -1,4 +1,3 @@
-from allauth.socialaccount import providers
 from allauth.socialaccount.providers.base import ProviderAccount
 from allauth.socialaccount.providers.oauth.provider import OAuthProvider
 
@@ -12,16 +11,15 @@ class EvernoteAccount(ProviderAccount):
 
 
 class EvernoteProvider(OAuthProvider):
-    id = 'evernote'
-    name = 'Evernote'
-    package = 'allauth.socialaccount.providers.evernote'
+    id = "evernote"
+    name = "Evernote"
     account_class = EvernoteAccount
 
     def extract_uid(self, data):
-        return str(data['edam_userId'])
+        return str(data["edam_userId"])
 
     def extract_common_fields(self, data):
         return data
 
 
-providers.registry.register(EvernoteProvider)
+provider_classes = [EvernoteProvider]
