@@ -1,5 +1,7 @@
 import json
+from unittest.mock import patch
 
+from django.contrib.auth import get_user_model
 from django.test.client import RequestFactory
 from django.test.utils import override_settings
 from django.urls import reverse
@@ -8,8 +10,7 @@ from allauth.account import app_settings as account_settings
 from allauth.account.models import EmailAddress
 from allauth.socialaccount.models import SocialAccount
 from allauth.socialaccount.tests import OAuth2TestsMixin
-from allauth.tests import MockedResponse, TestCase, patch
-from allauth.utils import get_user_model
+from allauth.tests import MockedResponse, TestCase
 
 from .provider import FacebookProvider
 
@@ -89,8 +90,17 @@ class FacebookTests(OAuth2TestsMixin, TestCase):
         with patch(
             "allauth.socialaccount.providers.facebook.views.requests"
         ) as requests_mock:
-            mocks = [self.get_mocked_response().json()]
-            requests_mock.get.return_value.json = lambda: mocks.pop()
+            mocks = [
+                {"access_token": "app_token"},
+                {
+                    "data": {
+                        "app_id": "app123id",
+                        "is_valid": True,
+                    }
+                },
+                self.get_mocked_response().json(),
+            ]
+            requests_mock.get.return_value.json = lambda: mocks.pop(0)
             resp = self.client.post(
                 reverse("facebook_login_by_token"),
                 data={"access_token": "dummy"},
@@ -114,8 +124,18 @@ class FacebookTests(OAuth2TestsMixin, TestCase):
         with patch(
             "allauth.socialaccount.providers.facebook.views.requests"
         ) as requests_mock:
-            mocks = [self.get_mocked_response().json(), {"auth_nonce": nonce}]
-            requests_mock.get.return_value.json = lambda: mocks.pop()
+            mocks = [
+                {"access_token": "app_token"},
+                {
+                    "data": {
+                        "app_id": "app123id",
+                        "is_valid": True,
+                    }
+                },
+                {"auth_nonce": nonce},
+                self.get_mocked_response().json(),
+            ]
+            requests_mock.get.return_value.json = lambda: mocks.pop(0)
             resp = self.client.post(
                 reverse("facebook_login_by_token"),
                 data={"access_token": "dummy"},
