@@ -1,4 +1,5 @@
 import django
+from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
 from django.contrib.messages.middleware import MessageMiddleware
 from django.contrib.sessions.middleware import SessionMiddleware
@@ -10,12 +11,12 @@ import allauth.app_settings
 from allauth.account import app_settings as account_settings
 from allauth.account.models import EmailAddress
 from allauth.account.utils import user_email, user_username
+from allauth.core import context
 from allauth.socialaccount import providers
 from allauth.socialaccount.helpers import complete_social_login
 from allauth.socialaccount.models import SocialAccount, SocialApp, SocialLogin
 from allauth.socialaccount.views import signup
 from allauth.tests import TestCase
-from allauth.utils import get_user_model
 
 
 class SignupTests(TestCase):
@@ -56,7 +57,8 @@ class SignupTests(TestCase):
 
         account = SocialAccount(provider="openid", uid="123")
         sociallogin = SocialLogin(user=user, account=account)
-        complete_social_login(request, sociallogin)
+        with context.request_context(request):
+            complete_social_login(request, sociallogin)
 
         user = User.objects.get(**{account_settings.USER_MODEL_USERNAME_FIELD: "test"})
         self.assertTrue(
@@ -81,7 +83,8 @@ class SignupTests(TestCase):
         # POST different username/email to social signup form
         request.method = "POST"
         request.POST = {"username": "other", "email": "other@example.com"}
-        resp = signup(request)
+        with context.request_context(request):
+            resp = signup(request)
         self.assertEqual(resp["location"], "/accounts/profile/")
         user = get_user_model().objects.get(
             **{account_settings.USER_MODEL_EMAIL_FIELD: "other@example.com"}
@@ -103,7 +106,8 @@ class SignupTests(TestCase):
         # POST email to social signup form (username not present)
         request.method = "POST"
         request.POST = {"email": "other@example.com"}
-        resp = signup(request)
+        with context.request_context(request):
+            resp = signup(request)
         self.assertEqual(resp["location"], "/accounts/profile/")
         user = get_user_model().objects.get(
             **{account_settings.USER_MODEL_EMAIL_FIELD: "other@example.com"}
@@ -152,7 +156,8 @@ class SignupTests(TestCase):
 
         account = SocialAccount(provider="twitter", uid="123")
         sociallogin = SocialLogin(user=user, account=account)
-        complete_social_login(request, sociallogin)
+        with context.request_context(request):
+            complete_social_login(request, sociallogin)
 
         self.assertNotIn(request.user.username, account_settings.USERNAME_BLACKLIST)
 
@@ -181,28 +186,10 @@ class SignupTests(TestCase):
         request.user = AnonymousUser()
         SessionMiddleware(lambda request: None).process_request(request)
         MessageMiddleware(lambda request: None).process_request(request)
-        resp = complete_social_login(request, sociallogin)
+        with context.request_context(request):
+            resp = complete_social_login(request, sociallogin)
         return request, resp
 
-    def test_disconnect(self):
-        User = get_user_model()
-        # Some existig user
-        user = User()
-        user_username(user, "test")
-        user_email(user, "test@example.com")
-        user.set_password("test")
-        user.save()
-
-        account = SocialAccount.objects.create(uid="123", provider="twitter", user=user)
-
-        self.client.login(username=user.username, password=user.username)
-        resp = self.client.get(reverse("socialaccount_connections"))
-        self.assertTemplateUsed(resp, "socialaccount/connections.html")
-        resp = self.client.post(
-            reverse("socialaccount_connections"), {"account": account.pk}
-        )
-        self.assertFalse(SocialAccount.objects.filter(pk=account.pk).exists())
-
     @override_settings(
         ACCOUNT_EMAIL_REQUIRED=True,
         ACCOUNT_EMAIL_VERIFICATION="mandatory",
@@ -483,3 +470,38 @@ def test_email_address_conflict_during_auto_signup(
     resp = complete_social_login(request, sociallogin)
     assert resp["location"] == reverse("account_email_verification_sent")
     assert mailoutbox[0].subject == "[example.com] Account Already Exists"
+
+
+def test_email_address_conflict_removes_conflicting_email(
+    db, settings, user_factory, sociallogin_factory, client, rf, mailoutbox
+):
+    """Tests that when an already existing email is given at the social signup
+    form, enumeration preventation kicks in.
+    """
+    settings.ACCOUNT_EMAIL_REQUIRED = True
+    settings.ACCOUNT_UNIQUE_EMAIL = True
+    settings.ACCOUNT_USERNAME_REQUIRED = False
+    settings.ACCOUNT_AUTHENTICATION_METHOD = "email"
+    settings.ACCOUNT_EMAIL_VERIFICATION = "optional"
+    settings.SOCIALACCOUNT_AUTO_SIGNUP = True
+    settings.SOCIALACCOUNT_EMAIL_AUTHENTICATION = False
+
+    user = user_factory(email_verified=False)
+    sociallogin = sociallogin_factory(email=user.email, email_verified=False)
+
+    request = rf.get("/")
+    request.session = {}
+    request.user = AnonymousUser()
+
+    resp = complete_social_login(request, sociallogin)
+    # Auto signup does not kick in as the `sociallogin` has a conflicting email.
+    assert resp["location"] == reverse("socialaccount_signup")
+
+    session = client.session
+    session["socialaccount_sociallogin"] = sociallogin.serialize()
+    session.save()
+    # Here, we input the already existing email.
+    resp = client.post(reverse("socialaccount_signup"), {"email": "other@email.org"})
+    assert mailoutbox[0].subject == "[example.com] Please Confirm Your Email Address"
+    assert resp["location"] == settings.LOGIN_REDIRECT_URL
+    assert EmailAddress.objects.filter(email=user.email).count() == 1
