@@ -21,6 +21,7 @@ DATABASES = {
 }
 
 ROOT_URLCONF = "allauth.urls"
+LOGIN_URL = "/login/"
 
 TEMPLATES = [
     {
@@ -40,6 +41,12 @@ TEMPLATES = [
     },
 ]
 
+CACHES = {
+    "default": {
+        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
+    }
+}
+
 MIDDLEWARE = (
     "django.contrib.sessions.middleware.SessionMiddleware",
     "django.middleware.common.CommonMiddleware",
@@ -47,6 +54,7 @@ MIDDLEWARE = (
     "django.contrib.auth.middleware.AuthenticationMiddleware",
     "django.contrib.messages.middleware.MessageMiddleware",
     "django.middleware.clickjacking.XFrameOptionsMiddleware",
+    "allauth.account.middleware.AccountMiddleware",
 )
 
 INSTALLED_APPS = (
@@ -59,6 +67,7 @@ INSTALLED_APPS = (
     "django.contrib.admin",
     "allauth",
     "allauth.account",
+    "allauth.mfa",
     "allauth.socialaccount",
     "allauth.socialaccount.providers.agave",
     "allauth.socialaccount.providers.amazon",
@@ -68,7 +77,6 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.asana",
     "allauth.socialaccount.providers.auth0",
     "allauth.socialaccount.providers.authentiq",
-    "allauth.socialaccount.providers.azure",
     "allauth.socialaccount.providers.baidu",
     "allauth.socialaccount.providers.basecamp",
     "allauth.socialaccount.providers.battlenet",
@@ -76,7 +84,6 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.bitbucket_oauth2",
     "allauth.socialaccount.providers.bitly",
     "allauth.socialaccount.providers.box",
-    "allauth.socialaccount.providers.cern",
     "allauth.socialaccount.providers.cilogon",
     "allauth.socialaccount.providers.coinbase",
     "allauth.socialaccount.providers.dataporten",
@@ -116,7 +123,6 @@ INSTALLED_APPS = (
     "allauth.socialaccount.providers.instagram",
     "allauth.socialaccount.providers.jupyterhub",
     "allauth.socialaccount.providers.kakao",
-    "allauth.socialaccount.providers.keycloak",
     "allauth.socialaccount.providers.lemonldap",
     "allauth.socialaccount.providers.line",
     "allauth.socialaccount.providers.linkedin",
@@ -214,7 +220,7 @@ PASSWORD_HASHERS = [
 ACCOUNT_EMAIL_CONFIRMATION_COOLDOWN = 0
 ACCOUNT_RATE_LIMITS = {}
 
-
+SOCIALACCOUNT_QUERY_EMAIL = True
 SOCIALACCOUNT_PROVIDERS = {
     "openid_connect": {
         "APPS": [
